/**************************************************************************************************
 * Debian BIND Zone Wizard, Started 2-08-2000                                                     *
 *                                                                                                *
 * Command line tool for configuring the BIND 8+ name server daemon on Debian Linux systems.      *
 *                                                                                                *
 *                                                                                                *
 * Aaron Faby, afaby@amermfg.com                                                                  *
 * Copyright 2000, AmericanManufacturers.com                                                      *
 * http://www.americanmanufacturers.com                                                           *
 *                                                                                                *
 * This software is licensed under the GNU General Public License. You should have received a     *
 * copy of this license with this source code, however if you have not than you can get one at    *
 * http://www.gnu.org. This software is FREE and therefore has NO WARRANTY EXPRESSED OR IMPLIED.  *
 * AmericanManufacturers.com does not take responsibility for damages incurred from it's use.     *
 * USE AT YOUR OWN RISK.                                                                          *
 *                                                                                                *
 **************************************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#define VERSION "1.0"
#define BOOTOPTIONS "/var/named/boot.options"
#define BOOTZONES   "/var/named/boot.zones"
#define NAMEDCONF   "/etc/named.conf"

/* variable declarations */

struct p_zone {
    char zone[32];
    char email[40];
    int serial;
    int refresh;
    int retry;
    int expire;
    int ttl;
} primary_zone;

struct p_zone_data {
    char primary_dns[64];
    char secondary_dns[64];
    int mx_priority;
    char mx_host[64];
} primary_zone_data;

struct s_zone {
    char zone[32];
    char master[16];
} secondary_zone;

/* function declarations */

void main_menu(void);
void add_new_primary_zone(void);
void display_primary_zone_info(void);
void modify_primary_zone_info(int choice);
void add_new_primary_zone_data(void);
void display_primary_zone_data(void);
void modify_primary_zone_data(int choice);
void save_primary_zone_to_disk(void);
void restart_bind(void);
void add_new_secondary_zone(void);
void display_secondary_zone_info(void);
void modify_secondary_zone_info(int choice);
void save_secondary_zone_to_disk(void);

/* main starts the program */

int main(int argc, char *argv[])
{
    main_menu();

    return;
}

/* main_menu displays the program's main menu */

void main_menu(void)
{
    int choice; 

    /* Introduction and title sentence. */

    printf("\nDebian BIND Zone Creator, Version %s\n", VERSION);
    printf("Copyright, 2000 Aaron Faby\n");
    printf("\n1) Add a new primary zone.");
    printf("\n2) Add a new secondary zone.");
    printf("\n3) Restart named.");
    printf("\n4) Quit");

    /* Take user input using a fgets */

    printf("\n\nPlease choose and option: ");
    scanf("%d", &choice);

    getc(stdin);


    /* Use a case statement to execute the function corresponding to the user's choice */

    switch(choice)
    {
        case 1: add_new_primary_zone();            /* A choice of 1 will execute add_new_zone */
                break;

        case 2: add_new_secondary_zone();
                break;

        case 3: restart_bind();
                break;

        case 4: exit(0);                    /* A choice of 2 will exit the program */
                break;

        default: return;                   /* should the user choose anything else, the program will exit */
    }
}

/* add_new_primary_zone sets up primary zone information */

void add_new_primary_zone(void)
{
    int choice;
    int n;

    primary_zone.serial = 1;
    primary_zone.refresh = 10800;
    primary_zone.retry = 3600;
    primary_zone.expire = 604800;
    primary_zone.ttl = 86400;

    /* clear out any old strings in zone and email */

    for(n=0; n < 32; n++)
    {
        primary_zone.zone[n] = NULL;
    }

    for(n=0; n < 40; n++)
    {
        primary_zone.email[n] = NULL;
    }

    printf("\n\nAdd A New Primary Zone");
    printf("\n\nThe zone name is also known as the domain name. Please enter your");
    printf("\nnew domain name below. Remember that domain names are limited to 26");
    printf("\ncharacters, not including the TLD (.com, .net, etc.).");

    printf("\n\nEnter the name for the new primary zone: ");
    fgets(primary_zone.zone, 31, stdin);

    /* remove the newline character from the string */

    for(n=0; n < 32; n++)
    {
        if(primary_zone.zone[n] == '\n')
            primary_zone.zone[n] = '\0';
    }

    /* Now collect the email address of the responsible person for the domain */

    printf("\n\nEnter the email address of the person responsible for this zone: ");
    fgets(primary_zone.email, 39, stdin);

    /* remove the newline character from the string */

    for(n=0; n < 40; n++)
    {
        if(primary_zone.email[n] == '\n')
            primary_zone.email[n] = '\0';
        else if(primary_zone.email[n] == '@')
            primary_zone.email[n] = '.';
    }

    /* next we will grab some information for the SOA record */

    printf("\n\nNow we need some information for the SOA (start of authority record) for");
    printf("\nthis zone. The default values are in brackets.");

    printf("\n\nEnter the serial number for this record [%d]: ", primary_zone.serial);

    if((scanf("%d", &primary_zone.serial)) <= 0)
        primary_zone.serial = 1;

    /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

    getc(stdin);

    printf("\n\nPlease enter the refresh value in seconds [%d]: ", primary_zone.refresh);

    if((scanf("%d", &primary_zone.refresh)) < 0)
        primary_zone.refresh = 0;

    /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

    getc(stdin);

    printf("\n\nPlease enter the retry value in seconds [%d]: ", primary_zone.retry);

    if((scanf("%d", &primary_zone.retry)) < 0)
        primary_zone.retry = 0;

    /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

    getc(stdin);

    printf("\n\nPlease enter the expire value in seconds [%d]: ", primary_zone.expire);

    if((scanf("%d", &primary_zone.expire)) < 0)
        primary_zone.expire = 0;

    /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

    getc(stdin);

    printf("\n\nPlease enter the Time To Live value in seconds [%d]: ", primary_zone.ttl);

    if((scanf("%d", &primary_zone.ttl)) < 0)
        primary_zone.ttl = 0;

    /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

    getc(stdin);

    display_primary_zone_info();
    
    return;
}   

/* display_primary_zone_info displays the user's info and gives them the choice to edit it before commiting */

void display_primary_zone_info(void)
{
    int choice;

    printf("\n\n************************************");
    printf("\n* Primary Zone Information Summary *");
    printf("\n************************************");
    printf("\n\nZone Name: %s", primary_zone.zone);
    printf("\nEmail Address: %s", primary_zone.email);
    printf("\nSerial Number: %d", primary_zone.serial);
    printf("\nRefresh Value: %d", primary_zone.refresh);
    printf("\nRetry Value: %d", primary_zone.retry);
    printf("\nExpire Value: %d", primary_zone.expire);
    printf("\nTTL Value: %d", primary_zone.ttl);
    printf("\n\n1) Modify Zone Name");
    printf("\n2) Modify Email Address");
    printf("\n3) Modify Serial Number");
    printf("\n4) Modify Refresh Value");
    printf("\n5) Modify Retry Value");
    printf("\n6) Modify Expire Value");
    printf("\n7) Modify TTL Value");
    printf("\n8) Keep Values And Continue");
    printf("\n9) Back To Main Menu");
    printf("\n\nEnter choice: ");

    scanf("%d", &choice);

    /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

    getc(stdin);

    modify_primary_zone_info(choice);
}

/* modify_primary_zone_info actually performs the modifying of values based on user input */

void modify_primary_zone_info(int choice)
{
    int n; 
   
    if(choice > 9 || choice <= 0)
        display_primary_zone_info();

    switch(choice)
    {
        case 1: printf("\n\nEnter new zone name: ");
                fgets(primary_zone.zone, 31, stdin);

                /* remove the newline character from the string */

                for(n=0; n < 32; n++)
                {
                    if(primary_zone.zone[n] == '\n')
                        primary_zone.zone[n] = '\0';
                }
      
                display_primary_zone_info();
                break;

        case 2: printf("\n\nEnter a new email address: ");
                fgets(primary_zone.email, 39, stdin);

                /* remove the newline character from the string */

                for(n=0; n < 40; n++)
                {
                    if(primary_zone.email[n] == '\n')
                        primary_zone.email[n] = '\0';
                }
      
                display_primary_zone_info();
                break;

        case 3: printf("\n\nEnter a new serial number: ");
                if((scanf("%d", &primary_zone.serial)) <= 0)
                    primary_zone.serial = 1;

                /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

                getc(stdin);

                display_primary_zone_info();
                break;

        case 4: printf("\n\nEnter a new refresh value: ");
                if((scanf("%d", &primary_zone.refresh)) <= 0)
                    primary_zone.refresh = 1;

                /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

                getc(stdin);

                display_primary_zone_info();
                break;

        case 5: printf("\n\nEnter a new retry value: ");
                if((scanf("%d", &primary_zone.retry)) <= 0)
                    primary_zone.retry = 1;

                /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

                getc(stdin);

                display_primary_zone_info();
                break;

        case 6: printf("\n\nEnter a new expire value: ");
                if((scanf("%d", &primary_zone.expire)) <= 0)
                    primary_zone.expire = 1;

                /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

                getc(stdin);

                display_primary_zone_info();
                break;

        case 7: printf("\n\nEnter a new TTL value: ");
                if((scanf("%d", &primary_zone.ttl)) <= 0)
                    primary_zone.ttl = 1;

                /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

                getc(stdin);

                display_primary_zone_info();
                break;

        case 8: add_new_primary_zone_data();              
                break;

        case 9: main_menu();
                break;

        default: display_primary_zone_info();
    }
}

void add_new_primary_zone_data(void)
{
    char choice;
    int n;

    printf("\n\nPrimary Zone Data");
    printf("\n=================");
    printf("\n\nWe would now like to set up some initial zone data for %s.", primary_zone.zone);
    printf("\nThis next step will set up your initial NS [name server] and MX [mail exchanger]");
    printf("\nrecords. It is recommended that you take this step. Please answer \'y\' or \'n\'.");
    printf("\n\nWould you like to add any initial primary zone data [y]? ");

    scanf("%c", &choice);

    getc(stdin);

    if(choice == 'y' || choice == 'Y')
    {
        printf("\n\nEnter the first Name Server for this domain [example: ns.domain.com]: ");
        fgets(primary_zone_data.primary_dns, 63, stdin);

        /* remove the newline character from the string */

        for(n=0; n < 64; n++)
        {
            if(primary_zone_data.primary_dns[n] == '\n')
                primary_zone_data.primary_dns[n] = '\0';
        }

        printf("\n\nEnter the second Name Server for this domain [example: ns.domain.com]: ");
        fgets(primary_zone_data.secondary_dns, 63, stdin);

        /* remove the newline character from the string */

        for(n=0; n < 64; n++)
        {
            if(primary_zone_data.secondary_dns[n] == '\n')
                primary_zone_data.secondary_dns[n] = '\0';
        }

        printf("\n\nPlease enter a numeric value for the priority of the MX record [10]: ");

        scanf("%d", &primary_zone_data.mx_priority);

        getc(stdin);

        printf("\n\nPlease enter the mail server for this record [mail.domain.com]: ");
        fgets(primary_zone_data.mx_host, 63, stdin);

        /* remove the newline character from the string */

        for(n=0; n < 64; n++)
        {
            if(primary_zone_data.mx_host[n] == '\n')
                primary_zone_data.mx_host[n] = '\0';
        }
    }
    else
        save_primary_zone_to_disk();

    display_primary_zone_data();

    return;
}

void display_primary_zone_data(void)
{
    int choice;

    printf("\n\n***********************************");
    printf("\n*    Primary Zone Data Summary    *");
    printf("\n***********************************");
    printf("\n\nPrimary Name Server: %s", primary_zone_data.primary_dns);
    printf("\nSecondary Name Server: %s", primary_zone_data.secondary_dns);
    printf("\nMail Exchanger Priority: %d", primary_zone_data.mx_priority);
    printf("\nMail Host: %s", primary_zone_data.mx_host);
    printf("\n\n1) Modify Primary Name Server");
    printf("\n2) Modify Secondary Name Server");
    printf("\n3) Mail Exchanger Priority");
    printf("\n4) Modify Mail Host");;
    printf("\n5) Keep Values And Save To Disk");
    printf("\n6) Back To Main Menu");
    printf("\n\nEnter choice: ");

    scanf("%d", &choice);

    /* grab the next character from the input stream so the newline will not carry over to our next scanf call */

    getc(stdin);

    modify_primary_zone_data(choice);
}

void modify_primary_zone_data(int choice)
{
    int n;

    if(choice <= 0 || choice > 6)
        display_primary_zone_data();

    switch(choice)
    {
        case 1: printf("\n\nEnter the first Name Server for this domain [example: ns.domain.com]: ");
                fgets(primary_zone_data.primary_dns, 63, stdin);

                /* remove the newline character from the string */

                for(n=0; n < 64; n++)
                {
                if(primary_zone_data.primary_dns[n] == '\n')
                    primary_zone_data.primary_dns[n] = '\0';
                }

                display_primary_zone_data();
                break;

        case 2: printf("\n\nEnter the second Name Server for this domain [example: ns.domain.com]: ");
                fgets(primary_zone_data.secondary_dns, 63, stdin);

                /* remove the newline character from the string */

                for(n=0; n < 64; n++)
                {
                    if(primary_zone_data.secondary_dns[n] == '\n')
                        primary_zone_data.secondary_dns[n] = '\0';
                }

                display_primary_zone_data();
                break;

        case 3: printf("\n\nPlease enter a numeric value for the priority of the MX record [10]: ");

                scanf("%d", &primary_zone_data.mx_priority);

                getc(stdin);

                display_primary_zone_data();
                break;

        case 4: printf("\n\nPlease enter the mail server for this record [mail.domain.com]: ");
                fgets(primary_zone_data.mx_host, 63, stdin);

                /* remove the newline character from the string */

                for(n=0; n < 64; n++)
                {
                    if(primary_zone_data.mx_host[n] == '\n')
                        primary_zone_data.mx_host[n] = '\0';
                }

                display_primary_zone_data();
                break;

        case 5: save_primary_zone_to_disk();
                break;

        case 6: main_menu();
                break;
    
        default: display_primary_zone_data();
    }
}

/* Save primary Zone information to disk. */

void save_primary_zone_to_disk(void)
{
    FILE *fp;
    int n;
    char choice;
    int end_choice;
    char *zone_file; 
    char named[58] = "/Local/Users/afaby/Development/Utilities/DNS/named/named.";

    printf("\n\nPreparing to save information to file.. Proceed? [y]: ");

    scanf("%c", &choice);

    getc(stdin);

    if(choice == 'y' || choice == 'Y')
    {
        if((fp=fopen(NAMEDCONF, "a"))==NULL)
        {
            printf("\n\nError opening file %s! Returning to main menu...", NAMEDCONF);
            fclose(fp);
            main_menu();
            return;
        }
        else
        {
            fprintf(fp, "\n\nzone \"%s\" {", primary_zone.zone);
            fprintf(fp, "\n\ttype master;");
            fprintf(fp, "\n\tfile \"named.%s\";", primary_zone.zone);
            fprintf(fp, "\n};");
            fclose(fp);
            sleep(1);
            printf("\n\nWrote data to named.conf...");
        }

        if((fp=fopen(BOOTOPTIONS, "a"))==NULL)
        {
            printf("\n\nError opening file %s! Returning to main menu...", BOOTOPTIONS);
            fclose(fp);
            main_menu();
            return;
        }
        else
        {
            fprintf(fp, "\nprimary\t\t%s\t\tnamed.%s", primary_zone.zone, primary_zone.zone);
            fclose(fp);
            sleep(1);
            printf("\n\nWrote data to boot.options...");
        }

        if((fp=fopen(BOOTZONES, "a"))==NULL)
        {
            printf("Error opening file %s! Returning to main menu...", BOOTZONES);
            fclose(fp);
            main_menu();
            return;
        }
        else
        {
            fprintf(fp, "\nprimary\t\t%s\t\tnamed.%s", primary_zone.zone, primary_zone.zone);
            fclose(fp);
            sleep(1);
            printf("\n\nWrote data to boot.zones...\n\n");
        }

        zone_file = (char *)strcat(named, primary_zone.zone);

        if((fp=fopen(zone_file, "w+"))==NULL)
        {
            printf("\n\nError creating file %s! Returning to main menu...", zone_file);
            fclose(fp);
            main_menu();
            return;
        }
        else
        {
            if(primary_zone_data.primary_dns[0] == NULL)
                fprintf(fp, "@\tIN SOA\t ns.%s. %s. (", primary_zone.zone, primary_zone.email);
            else
                fprintf(fp, "@\tIN SOA\t %s. %s. (", primary_zone_data.primary_dns, primary_zone.email);

            fprintf(fp, "\n\t\t\t%d\t\t; Serial", primary_zone.serial);
            fprintf(fp, "\n\t\t\t%d\t\t; Refresh", primary_zone.refresh);
            fprintf(fp, "\n\t\t\t%d\t\t; Retry", primary_zone.retry);
            fprintf(fp, "\n\t\t\t%d\t\t; Expire", primary_zone.expire);
            fprintf(fp, "\n\t\t\t%d )\t\t; Minimum TTL", primary_zone.ttl);
            fprintf(fp, "\n\n; Name servers");
            if(primary_zone_data.primary_dns[0] != NULL)
            fprintf(fp, "\n\tIN NS\t %s.", primary_zone_data.primary_dns);
            if(primary_zone_data.secondary_dns[0] != NULL)
            fprintf(fp, "\n\tIN NS\t %s.", primary_zone_data.secondary_dns);
            fprintf(fp, "\n\n; Localhost Record");
            fprintf(fp, "\nlocalhost\t\tIN A\t 127.0.0.1");
            if(primary_zone_data.mx_priority != NULL)
                fprintf(fp, "\n\n%s.\t\tIN MX \t%d \t%s.", primary_zone.zone, primary_zone_data.mx_priority, primary_zone_data.mx_host);
            fclose(fp);    
            sleep(1);
            printf("\n\nCreated zone file %s...\n\n", zone_file);
        }
    }
    else
    {
        display_primary_zone_info();
    }

    printf("\nData has been saved.");
    printf("\n\n1) Return to Main Menu");
    printf("\n2) Quit");
    printf("\n\nChoice: ");
    
    scanf("%d", &end_choice);
    
    getc(stdin);

    if(end_choice == 1)
    {
        main_menu();
        return;
    }
    else
        exit(0);
}

/* Restart the named daemon. */

void restart_bind(void)
{
    printf("\n\nSending named a SIGHUP...\n\n");
    system("killall -HUP named");
    sleep(2);
    main_menu();
}

void add_new_secondary_zone(void)
{
    int choice;
    int n;

    printf("\n\nAdd A New Secondary Zone");
    printf("\n========================");
    printf("\n\nPlease enter the name of the secondary zone (domain name) you wish");
    printf("\nto add below. You will also need to know the IP address of the master");
    printf("\nserver in which you would like your server to make zone transfers.");
    printf("\n\nEnter the name of your new secondary zone: ");

    fgets(secondary_zone.zone, 31, stdin);

    for(n=0; n < 32; n++)
    {
        if(secondary_zone.zone[n] == '\n')
            secondary_zone.zone[n] = '\0';
    }

    printf("\n\nNow enter the IP address of the master server for this zone: ");

    fgets(secondary_zone.master, 15, stdin);

    for(n=0; n < 16; n++)
    {
        if(secondary_zone.master[n] == '\n')
            secondary_zone.master[n] = '\0';
    }

    display_secondary_zone_info();
  
    return;
}

void display_secondary_zone_info(void)
{
    int choice;

    printf("\n\n**************************************");
    printf("\n*Secondary Zone Information Summary *");
    printf("\n**************************************");
    printf("\n\nZone Name: %s", secondary_zone.zone);
    printf("\nMaster Server: %s", secondary_zone.master);
    printf("\n\n1) Modify Zone Name");
    printf("\n2) Modify Master Server");
    printf("\n3) Keep Values and Save To Disk");
    printf("\n4) Back To Main Menu");
    printf("\n\nPlease enter a choice: ");
 
    scanf("%d", &choice);
  
    getc(stdin);

    modify_secondary_zone_info(choice);
}

void modify_secondary_zone_info(choice)
{
    int n;

    if(choice <= 0 || choice > 4)
        display_secondary_zone_info();

    switch(choice)
    {
        case 1: printf("\n\nEnter new secondary zone name: ");
                fgets(secondary_zone.zone, 31, stdin);

                for(n=0; n < 32; n++)
                {
                    if(secondary_zone.zone[n] == '\n')
                        secondary_zone.zone[n] = '\0';
                } 

                display_secondary_zone_info();
                break;

        case 2: printf("\n\nEnter new Master Server: ");
                printf("\n\nNow enter the IP address of the master server for this zone: ");

                fgets(secondary_zone.master, 15, stdin);

                for(n=0; n < 16; n++)
                {
                    if(secondary_zone.master[n] == '\n')
                        secondary_zone.master[n] = '\0';
                }

                display_secondary_zone_info();
                break;

        case 3: save_secondary_zone_to_disk();
                break;

        case 4: main_menu();
                break;

        default: display_secondary_zone_info();
    }
}

void save_secondary_zone_to_disk(void)
{

    FILE *fp;
    int n;
    char choice;
    int end_choice;
    char *zone_file;
    char named[58] = "/Local/Users/afaby/Development/Utilities/DNS/named/named.";

    printf("\n\nPreparing to save information to file.. Proceed? [y]: ");

    scanf("%c", &choice);

    getc(stdin);

    if(choice == 'y' || choice == 'Y')
    {
        if((fp=fopen(NAMEDCONF, "a"))==NULL)
        {
            printf("\n\nError opening file %s! Returning to main menu...", NAMEDCONF);
            fclose(fp);
            main_menu();
            return;
        }
        else
        {
            fprintf(fp, "\n\nzone \"%s\" {", secondary_zone.zone);
            fprintf(fp, "\n\ttype slave;");
            fprintf(fp, "\n\tfile \"named.%s\";", secondary_zone.zone);
            fprintf(fp, "\n\tmasters { %s; };", secondary_zone.master);
            fprintf(fp, "\n};");
            fclose(fp);
            sleep(1);
            printf("\n\nWrote data to named.conf...");
        }

        if((fp=fopen(BOOTOPTIONS, "a"))==NULL)
        {
            printf("\n\nError opening file %s! Returning to main menu...", BOOTOPTIONS);
            fclose(fp);
            main_menu();
            return;
        }
        else
        {
            fprintf(fp, "\nslave\t\t%s\t\t%s\t\tnamed.%s", secondary_zone.zone, secondary_zone.master, secondary_zone.zone);
            fclose(fp);
            sleep(1);
            printf("\n\nWrote data to boot.options...");
        }

        if((fp=fopen(BOOTZONES, "a"))==NULL)
        {
            printf("Error opening file %s! Returning to main menu...", BOOTZONES);
            fclose(fp);
            main_menu();
            return;
        }
        else
        {
            fprintf(fp, "\nslave\t\t%s\t\t%s\t\tnamed.%s", secondary_zone.zone, secondary_zone.master, secondary_zone.zone);        
            fclose(fp);
            sleep(1);
            printf("\n\nWrote data to boot.zones...\n\n");
        }

        zone_file = (char *)strcat(named, secondary_zone.zone);

        if((fp=fopen(zone_file, "w+"))==NULL)
        {
            printf("\n\nError creating file %s! Returning to main menu...", zone_file);
            fclose(fp);
            main_menu();
            return;
        }
        else
        {
            fclose(fp);
            sleep(1);
            printf("\n\nCreated zone file %s...\n\n", zone_file);
        }    
    }
    else
    {
        display_secondary_zone_info();
    }

    printf("\nData has been saved.");
    printf("\n\n1) Return to Main Menu");
    printf("\n2) Quit");
    printf("\n\nChoice: ");

    scanf("%d", &end_choice);

    getc(stdin);

    if(end_choice == 1)
    {
        main_menu();
        return;
    }
    else
        exit(0);
}
               
/***************
 * End of File *
 ***************/